@@ -4,32 +4,26 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 #include "GMPServiceChild.h"
-#include "mozilla/dom/ContentChild.h"
-#include "mozilla/ClearOnShutdown.h"
-#include "mozilla/StaticPtr.h"
-#include "mozIGeckoMediaPluginService.h"
-#include "mozIGeckoMediaPluginChromeService.h"
-#include "nsCOMPtr.h"
+
+#include "base/task.h"
+#include "GMPLog.h"
 #include "GMPParent.h"
 #include "GMPContentParent.h"
-#include "nsXPCOMPrivate.h"
-#include "mozilla/SyncRunnable.h"
+#include "mozIGeckoMediaPluginChromeService.h"
+#include "mozilla/ClearOnShutdown.h"
+#include "mozilla/dom/ContentChild.h"
+#include "mozilla/SchedulerGroup.h"
 #include "mozilla/StaticMutex.h"
-#include "runnable_utils.h"
-#include "base/task.h"
-#include "nsIObserverService.h"
+#include "mozilla/StaticPtr.h"
+#include "mozilla/SyncRunnable.h"
 #include "nsComponentManagerUtils.h"
-#include "mozilla/SystemGroup.h"
+#include "nsCOMPtr.h"
+#include "nsIObserverService.h"
+#include "nsXPCOMPrivate.h"
+#include "runnable_utils.h"
 
 namespace mozilla {
 
-#ifdef LOG
-#  undef LOG
-#endif
-
-#define LOGD(msg) MOZ_LOG(GetGMPLog(), mozilla::LogLevel::Debug, msg)
-#define LOG(level, msg) MOZ_LOG(GetGMPLog(), (level), msg)
-
 #ifdef __CLASS__
 #  undef __CLASS__
 #endif
@@ -65,12 +59,11 @@ GeckoMediaPluginServiceChild::GetContentParent(
 
   nsCString nodeIdString(aNodeIdString);
   nsCString api(aAPI);
-  nsTArray<nsCString> tags(aTags);
   RefPtr<GMPCrashHelper> helper(aHelper);
   RefPtr<GeckoMediaPluginServiceChild> self(this);
   GetServiceChild()->Then(
       thread, __func__,
-      [self, nodeIdString, api, tags, helper,
+      [self, nodeIdString, api, tags = aTags.Clone(), helper,
        rawHolder](GMPServiceChild* child) {
         UniquePtr<MozPromiseHolder<GetGMPContentParentPromise>> holder(
             rawHolder);
@@ -104,7 +97,8 @@ GeckoMediaPluginServiceChild::GetContentParent(
                       "SendLaunchGMPForNodeId failed with description (%s)",
                       errorDescription.get()));
 
-          LOGD(("%s", error.Description().get()));
+          GMP_LOG_DEBUG("%s failed to launch GMP with error: %s", __CLASS__,
+                        error.Description().get());
           holder->Reject(error, __func__);
           return;
         }
@@ -141,12 +135,12 @@ GeckoMediaPluginServiceChild::GetContentParent(
 
   NodeIdData nodeId(aNodeId.mOrigin, aNodeId.mTopLevelOrigin, aNodeId.mGMPName);
   nsCString api(aAPI);
-  nsTArray<nsCString> tags(aTags);
   RefPtr<GMPCrashHelper> helper(aHelper);
   RefPtr<GeckoMediaPluginServiceChild> self(this);
   GetServiceChild()->Then(
       thread, __func__,
-      [self, nodeId, api, tags, helper, rawHolder](GMPServiceChild* child) {
+      [self, nodeId, api, tags = aTags.Clone(), helper,
+       rawHolder](GMPServiceChild* child) {
         UniquePtr<MozPromiseHolder<GetGMPContentParentPromise>> holder(
             rawHolder);
         nsresult rv;
@@ -180,7 +174,8 @@ GeckoMediaPluginServiceChild::GetContentParent(
                       "SendLaunchGMPForNodeId failed with description (%s)",
                       errorDescription.get()));
 
-          LOGD(("%s", error.Description().get()));
+          GMP_LOG_DEBUG("%s failed to launch GMP with error: %s", __CLASS__,
+                        error.Description().get());
           holder->Reject(error, __func__);
           return;
         }
@@ -278,7 +273,8 @@ void GeckoMediaPluginServiceChild::UpdateGMPCapabilities(
       sGMPCapabilities->AppendElement(GMPCapabilityAndVersion(plugin));
     }
 
-    LOGD(("UpdateGMPCapabilities {%s}", GMPCapabilitiesToString().get()));
+    GMP_LOG_DEBUG("%s::%s {%s}", __CLASS__, __FUNCTION__,
+                  GMPCapabilitiesToString().get());
   }
 
   // Fire a notification so that any MediaKeySystemAccess
@@ -352,7 +348,7 @@ GeckoMediaPluginServiceChild::GetNodeId(
 NS_IMETHODIMP
 GeckoMediaPluginServiceChild::Observe(nsISupports* aSubject, const char* aTopic,
                                       const char16_t* aSomeData) {
-  LOGD(("%s::%s: %s", __CLASS__, __FUNCTION__, aTopic));
+  GMP_LOG_DEBUG("%s::%s: %s", __CLASS__, __FUNCTION__, aTopic);
   if (!strcmp(NS_XPCOM_SHUTDOWN_THREADS_OBSERVER_ID, aTopic)) {
     if (mServiceChild) {
       mozilla::SyncRunnable::DispatchToThread(
@@ -391,7 +387,7 @@ GeckoMediaPluginServiceChild::GetServiceChild() {
     if (mGetServiceChildPromises.Length() == 1) {
       nsCOMPtr<nsIRunnable> r =
           WrapRunnable(contentChild, &dom::ContentChild::SendCreateGMPService);
-      SystemGroup::Dispatch(TaskCategory::Other, r.forget());
+      SchedulerGroup::Dispatch(TaskCategory::Other, r.forget());
     }
     return promise;
   }
@@ -425,9 +421,9 @@ void GeckoMediaPluginServiceChild::RemoveGMPContentParent(
   }
 }
 
-GMPServiceChild::GMPServiceChild() {}
+GMPServiceChild::GMPServiceChild() = default;
 
-GMPServiceChild::~GMPServiceChild() {}
+GMPServiceChild::~GMPServiceChild() = default;
 
 already_AddRefed<GMPContentParent> GMPServiceChild::GetBridgedGMPContentParent(
     ProcessId aOtherPid, ipc::Endpoint<PGMPContentParent>&& endpoint) {
@@ -445,7 +441,7 @@ already_AddRefed<GMPContentParent> GMPServiceChild::GetBridgedGMPContentParent(
   DebugOnly<bool> ok = endpoint.Bind(parent);
   MOZ_ASSERT(ok);
 
-  mContentParents.Put(aOtherPid, parent);
+  mContentParents.Put(aOtherPid, RefPtr{parent});
 
   return parent.forget();
 }
@@ -529,3 +525,5 @@ bool GMPServiceChild::HaveContentParents() const {
 
 }  // namespace gmp
 }  // namespace mozilla
+
+#undef __CLASS__
