@@ -24,9 +24,8 @@ using media::TimeIntervals;
 using media::TimeUnit;
 
 MediaSourceDemuxer::MediaSourceDemuxer(AbstractThread* aAbstractMainThread)
-    : mTaskQueue(
-          new AutoTaskQueue(GetMediaThreadPool(MediaThreadType::PLAYBACK),
-                            "MediaSourceDemuxer::mTaskQueue")),
+    : mTaskQueue(new TaskQueue(GetMediaThreadPool(MediaThreadType::PLAYBACK),
+                               "MediaSourceDemuxer::mTaskQueue")),
       mMonitor("MediaSourceDemuxer") {
   MOZ_ASSERT(NS_IsMainThread());
 }
@@ -164,7 +163,7 @@ void MediaSourceDemuxer::AttachSourceBuffer(
 void MediaSourceDemuxer::DoAttachSourceBuffer(
     RefPtr<mozilla::TrackBuffersManager>&& aSourceBuffer) {
   MOZ_ASSERT(OnTaskQueue());
-  mSourceBuffers.AppendElement(Move(aSourceBuffer));
+  mSourceBuffers.AppendElement(std::move(aSourceBuffer));
   ScanSourceBuffersForContent();
 }
 
@@ -231,49 +230,14 @@ MediaSourceDemuxer::~MediaSourceDemuxer() {
   mInitPromise.RejectIfExists(NS_ERROR_DOM_MEDIA_CANCELED, __func__);
 }
 
-void MediaSourceDemuxer::GetMozDebugReaderData(nsACString& aString) {
+void MediaSourceDemuxer::GetDebugInfo(dom::MediaSourceDemuxerDebugInfo& aInfo) {
   MonitorAutoLock mon(mMonitor);
-  nsAutoCString result;
-  result += nsPrintfCString("Dumping Data for Demuxer: %p\n", this);
   if (mAudioTrack) {
-    result += nsPrintfCString(
-        "\tDumping Audio Track Buffer(%s): mLastAudioTime=%f\n"
-        "\t\tAudio Track Buffer Details: NumSamples=%zu"
-        " Size=%u Evictable=%u "
-        "NextGetSampleIndex=%u NextInsertionIndex=%d\n",
-        mAudioTrack->mType.Type().AsString().get(),
-        mAudioTrack->mAudioTracks.mNextSampleTime.ToSeconds(),
-        mAudioTrack->mAudioTracks.mBuffers[0].Length(),
-        mAudioTrack->mAudioTracks.mSizeBuffer,
-        mAudioTrack->Evictable(TrackInfo::kAudioTrack),
-        mAudioTrack->mAudioTracks.mNextGetSampleIndex.valueOr(-1),
-        mAudioTrack->mAudioTracks.mNextInsertionIndex.valueOr(-1));
-
-    result += nsPrintfCString(
-        "\t\tAudio Track Buffered: ranges=%s\n",
-        DumpTimeRanges(mAudioTrack->SafeBuffered(TrackInfo::kAudioTrack))
-            .get());
+    mAudioTrack->GetDebugInfo(aInfo.mAudioTrack);
   }
   if (mVideoTrack) {
-    result += nsPrintfCString(
-        "\tDumping Video Track Buffer(%s): mLastVideoTime=%f\n"
-        "\t\tVideo Track Buffer Details: NumSamples=%zu"
-        " Size=%u Evictable=%u "
-        "NextGetSampleIndex=%u NextInsertionIndex=%d\n",
-        mVideoTrack->mType.Type().AsString().get(),
-        mVideoTrack->mVideoTracks.mNextSampleTime.ToSeconds(),
-        mVideoTrack->mVideoTracks.mBuffers[0].Length(),
-        mVideoTrack->mVideoTracks.mSizeBuffer,
-        mVideoTrack->Evictable(TrackInfo::kVideoTrack),
-        mVideoTrack->mVideoTracks.mNextGetSampleIndex.valueOr(-1),
-        mVideoTrack->mVideoTracks.mNextInsertionIndex.valueOr(-1));
-
-    result += nsPrintfCString(
-        "\t\tVideo Track Buffered: ranges=%s\n",
-        DumpTimeRanges(mVideoTrack->SafeBuffered(TrackInfo::kVideoTrack))
-            .get());
+    mVideoTrack->GetDebugInfo(aInfo.mVideoTrack);
   }
-  aString += result;
 }
 
 MediaSourceTrackDemuxer::MediaSourceTrackDemuxer(MediaSourceDemuxer* aParent,
@@ -443,7 +407,7 @@ MediaSourceTrackDemuxer::DoGetSamples(int32_t aNumSamples) {
     TimeIntervals buffered = mManager->Buffered(mType);
     buffered.SetFuzz(MediaSourceDemuxer::EOS_FUZZ / 2);
 
-    if (!buffered.Length() && mManager->IsEnded()) {
+    if (buffered.IsEmpty() && mManager->IsEnded()) {
       return SamplesPromise::CreateAndReject(NS_ERROR_DOM_MEDIA_END_OF_STREAM,
                                              __func__);
     }
@@ -473,7 +437,7 @@ MediaSourceTrackDemuxer::DoGetSamples(int32_t aNumSamples) {
     }
   }
   RefPtr<SamplesHolder> samples = new SamplesHolder;
-  samples->mSamples.AppendElement(sample);
+  samples->AppendSample(sample);
   if (mNextRandomAccessPoint <= sample->mTime) {
     MonitorAutoLock mon(mMonitor);
     mNextRandomAccessPoint =
