@@ -11,9 +11,9 @@ namespace mozilla {
 namespace media {
 
 MediaSystemResourceManagerChild::MediaSystemResourceManagerChild()
-    : mDestroyed(false) {}
+    : mDestroyed(false), mManager(nullptr) {}
 
-MediaSystemResourceManagerChild::~MediaSystemResourceManagerChild() {}
+MediaSystemResourceManagerChild::~MediaSystemResourceManagerChild() = default;
 
 mozilla::ipc::IPCResult MediaSystemResourceManagerChild::RecvResponse(
     const uint32_t& aId, const bool& aSuccess) {
