@@ -1,4 +1,4 @@
-/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 4 -*- */
+/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: nil; c-basic-offset: 2 -*- */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
@@ -14,17 +14,16 @@
 
 #include "nsCOMPtr.h"
 #include "nsTArray.h"
-#include "nsWeakPtr.h"
 #include "nsIControllers.h"
 #include "nsCycleCollectionParticipant.h"
 
 /* non-XPCOM class for holding controllers and their IDs */
-class nsXULControllerData {
+class nsXULControllerData final {
  public:
   nsXULControllerData(uint32_t inControllerID, nsIController* inController)
       : mControllerID(inControllerID), mController(inController) {}
 
-  ~nsXULControllerData() {}
+  ~nsXULControllerData() = default;
 
   uint32_t GetControllerID() { return mControllerID; }
 
@@ -37,20 +36,15 @@ class nsXULControllerData {
   nsCOMPtr<nsIController> mController;
 };
 
-nsresult NS_NewXULControllers(nsISupports* aOuter, REFNSIID aIID,
-                              void** aResult);
-
-class nsXULControllers : public nsIControllers {
+class nsXULControllers final : public nsIControllers {
  public:
-  friend nsresult NS_NewXULControllers(nsISupports* aOuter, REFNSIID aIID,
-                                       void** aResult);
+  nsXULControllers();
 
   NS_DECL_CYCLE_COLLECTING_ISUPPORTS
   NS_DECL_CYCLE_COLLECTION_CLASS_AMBIGUOUS(nsXULControllers, nsIControllers)
   NS_DECL_NSICONTROLLERS
 
  protected:
-  nsXULControllers();
   virtual ~nsXULControllers(void);
 
   void DeleteControllers();
